#ifdef _WIN32

#include "dataudiolib.h"
#include "if_userlevel.h"
#include "mt_ext.h"

#include "generic_scsi.h"
#include "win32.h"
#include "fd_win32.h"
#include <windows.h>

#define FRAMESIZE 5822

static int testtape(int fd) {
	unsigned char cdb[10];
	unsigned char buf[36];

	memset(cdb,0,6);
	cdb[0]=0x12;
	cdb[4]=36;
	fprintf(stderr,"Inquiry: ");
	if (send_scsi_command(fd,cdb,6,buf,36,NULL,0,NULL,0)<=0) {
		fprintf(stderr,"failed.\n");
		return -1;
		}
	else
		fprintf(stderr,"%.8s %.16s %.4s\n",buf+8,buf+16,buf+32);

	memset(cdb,0,6);
	fprintf(stderr,"Test unit ready: ");
	if (send_scsi_command(fd,cdb,6,NULL,0,NULL,0,NULL,0)<=0) {
		fprintf(stderr,"failed.\n");
		return -1;
		}
	else
		fprintf(stderr,"ok.\n");

#if 0
	/* non-essential tests */

	memset(cdb,0,6);
	cdb[0]=0x15;
	cdb[4]=0x0C;
	memset(buf,0,12);
	buf[2]=0x10;
	buf[3]=0x08;
	buf[10]=FRAMESIZE >> 8;
	buf[11]=FRAMESIZE & 0xFF;
	fprintf(stderr,"Mode select...");
	if (send_scsi_command(fd,cdb,6,NULL,0,buf,12,NULL,0)<0)
		fprintf(stderr," failed.\n");
	else
		fprintf(stderr," ok.\n");

	memset(cdb,0,6);
	cdb[0]=0x1B;
	cdb[4]=0x01;
	fprintf(stderr,"Load...");
	if (send_scsi_command(fd,cdb,6,NULL,0,NULL,0,NULL,0)<0)
		fprintf(stderr," failed.\n");
	else
		fprintf(stderr," ok.\n");

	memset(cdb,0,6);
	cdb[0]=0x01;
	fprintf(stderr,"Rewind...");
	if (send_scsi_command(fd,cdb,6,NULL,0,NULL,0,NULL,0)<0)
		fprintf(stderr," failed.\n");
	else
		fprintf(stderr," ok.\n");

	memset(cdb,0,10);
	cdb[0]=0x2B;
	fprintf(stderr,"Locate...");
	if (send_scsi_command(fd,cdb,10,NULL,0,NULL,0,NULL,0)<0)
		fprintf(stderr," failed.\n");
	else
		fprintf(stderr," ok.\n");

	memset(cdb,0,6);
	cdb[0]=0x05;
	fprintf(stderr,"Block limits...");
	if (send_scsi_command(fd,cdb,6,buf,6,NULL,0,NULL,0)<0)
		fprintf(stderr," failed.\n");
	else
		fprintf(stderr," 0x%02X%02X%02X to 0x%02X%02X%02X bytes\n",
			0,buf[4],buf[5],buf[1],buf[2],buf[3]);
#endif

	return 0;
	}

int da_downopen(char *fn,int opflags) {
	HANDLE h;
	int fd;

	h=CreateFile(fn,GENERIC_READ|GENERIC_WRITE,
		FILE_SHARE_READ|FILE_SHARE_WRITE,NULL,OPEN_EXISTING,
		FILE_FLAG_WRITE_THROUGH|FILE_FLAG_NO_BUFFERING,INVALID_HANDLE_VALUE);

	if (h==INVALID_HANDLE_VALUE) return -1;
	fd=fdw32_newfd(h);
	if (testtape(fd)>=0) return fd; /* good exit */
	fdw32_deletefd(fd);
	CloseHandle(h);
	return -1;
	}

int da_downinit(int fd,int opflags,char *fn,struct da_state *st) {
	struct	da_user_private *private;

	if (fd<0) return -1;
	private=malloc(sizeof(*private));
	if (!private) return -1;
	st->ifdata=(caddr_t)private;
	private->fn=w32_strdup(fn);
	private->type=DATAUDIO_FILE;
	return fd;
	}

int da_downclose(int fd) {
	HANDLE h;
	struct	da_state	*st;
	struct	da_user_private *private;
	extern	struct da_state *da_lookup(int id);
	
	st=da_lookup(fd);
	if (st==NULL) {fprintf(stderr,"da_lookup didn't find %d?\n",fd);exit(1);}
	private=(struct da_user_private*)st->ifdata;
	do {
		if (!private) break;
		if (private->fn) free(private->fn);
		free(private);
		} while(0);
	h=fdw32_gethandle(fd);
	fdw32_deletefd(fd);
	return CloseHandle(h);
	}

int da_downwrite(int fd,unsigned char *buf,int size) {
	return -1; /* lets leave this one until after testing */
	}

int da_downread(int fd,unsigned char *buf,int size) {
#if 1
	unsigned char cdb[6];
	cdb[0]=0x08; /* read */
	if (0 && (size % FRAMESIZE)==0) {
		cdb[1]=0x01;
		cdb[2]=0x00;
		cdb[3]=(size/FRAMESIZE)>>8;
		cdb[4]=(size/FRAMESIZE) & 0xFF;
		}
	else {
		cdb[1]=0x00;
		cdb[2]=(size>>16) & 255;
		cdb[3]=(size>>8) & 255;
		cdb[4]=(size>>0) & 255;
		}
	cdb[5]=0;
	if (send_scsi_command(fd,cdb,6,buf,size,NULL,0,NULL,0)<=0)
		return -1;

	return size;
#else
	BOOL r;
	DWORD bytes;
	r=ReadFile(fdw32_gethandle(fd),buf,size,&bytes,NULL);
	if (!r) return -1;
	return bytes;
#endif
	}

/* the UNIX ioctl interface, passes all necessary calls further down */
int da_downcontrol(int fd,int code,void *arg) {
	struct	da_locate	*daloc=arg;
	struct	da_position	*dapos;
	unsigned char cdb[10];
	unsigned	char	buf[20];
	int			res;
	struct da_state		*st;
	struct da_user_private	*priv;
	int oldtimeout;

	memset(cdb,0,sizeof(cdb));
	if (code==DA_LOCATE) {
		int t;
		st=da_lookup(fd);
		priv=(struct da_user_private*)st->ifdata;
		cdb[0]=0x2B; /* LOCATE */
		cdb[1]=0x04; /* BT=vendor specific */
		cdb[2]=0x00;
		cdb[4]=l2bcd(daloc->u.time.hour);
		cdb[5]=l2bcd(daloc->u.time.minute);
		cdb[6]=l2bcd(daloc->u.time.second);
		cdb[7]=l2bcd(daloc->u.time.frame);
		cdb[8]=0x00;
		cdb[9]=0x00;
		switch (daloc->type) {
			case DA_LOC_PROGNR:
				t=l2bcd(daloc->u.prognr);
				cdb[3]=0x00; /* prog nr */
				cdb[4]=(t>>8)&0x0F;
				cdb[5]=(t>>4)&0x0F;
				cdb[6]=(t>>0)&0x0F;
				cdb[7]=0x00;
				break;
			case DA_LOC_PROGTIME:
				cdb[3]=0x60; /* ptime */
				break;
			case DA_LOC_ABSTIME:
				cdb[3]=0x20; /* atime */
				break;
			case DA_LOC_RECTIME:
				cdb[3]=0x40; /* rtime */
				break;
			}
		res=send_scsi_command(fd,cdb,10,NULL,0,NULL,0,NULL,0);
		if (res==-1) {
			int	saveerrno=errno;
			perror("ioctl MTIOLOCATE");
			errno=saveerrno;
		}
		return res;
	}
	if (code==DA_READ_POSITION) {
		cdb[0]=0x34; /* READ POSITION */
		cdb[1]=0x01; /* vendor specific */
		res=send_scsi_command(fd,cdb,10,buf,20,NULL,0,NULL,0);
		if (res==-1) {
			perror("ioctl MTIOREADPOS");
			return -1;
		}
		dapos=(struct da_position*)arg;
		dapos->prognr=buf[4]*256+buf[5]*16+buf[6];
		dapos->indexnr=buf[7];
		dapos->progtime.indexnr=dapos->indexnr;
		dapos->abstime.indexnr=dapos->indexnr;
		dapos->runtime.indexnr=dapos->indexnr;
		dapos->progtime.hour=buf[8];
		dapos->progtime.minute=buf[9];
		dapos->progtime.second=buf[10];
		dapos->progtime.frame=buf[11];
		dapos->abstime.hour=buf[12];
		dapos->abstime.minute=buf[13];
		dapos->abstime.second=buf[14];
		dapos->abstime.frame=buf[15];
		dapos->runtime.hour=buf[16];
		dapos->runtime.minute=buf[17];
		dapos->runtime.second=buf[18];
		dapos->runtime.frame=buf[19];
		return 0;
		}
	if (code==DA_EJECT) {
		cdb[0]=0x1B;
		cdb[4]=0x00; /* 0x01 for load */
		fprintf(stderr,"Ejecting... ");
		oldtimeout=send_scsi_settimeoutms(45000);
		if (send_scsi_command(fd,cdb,6,NULL,0,NULL,0,NULL,0)<=0)
			fprintf(stderr,"failed.\n");
		else
			fprintf(stderr,"ok.\n");
		send_scsi_settimeoutms(oldtimeout);
		}
	if (code==DA_REWIND) {
		cdb[0]=0x01;
		fprintf(stderr,"Rewinding... ");
		oldtimeout=send_scsi_settimeoutms(120000);
		if (send_scsi_command(fd,cdb,6,NULL,0,NULL,0,NULL,0)<=0)
			fprintf(stderr,"failed.\n");
		else
			fprintf(stderr,"ok.\n");
		send_scsi_settimeoutms(oldtimeout);
		}
	if (code==DA_ABORT) {
		return -1;
		}
	return -1;
}

/*
MTIOLOCATE
MTIOREADPOS
MTIOCTOP: MTREW, MTOFFL (eject), etc.
MTIOABORT
MTIORESET
MTIOCGET
MTIOCGETDRIVETYPE
MTIOEXVER
MTIOQLPAGE
MTIOSMPAGE
MTIOQMPAGE
MTIOINQUIRY
*/

#endif
